package com.xkrs.controller;

import com.xkrs.common.encapsulation.PromptMessageEnum;
import com.xkrs.common.tool.TokenUtil;
import com.xkrs.dao.SysUserDao;
import com.xkrs.dao.WorkHourRecordDao;
import com.xkrs.model.entity.SysUserEntity;
import com.xkrs.model.entity.WorkHourRecord;
import com.xkrs.model.qo.WorkRecordQo;
import com.xkrs.model.vo.MemberWorkRecordVo;
import com.xkrs.model.vo.OutMemberVo;
import com.xkrs.service.WorkHourRecordService;
import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;
import java.util.Map;

import static com.xkrs.common.encapsulation.OutputEncapsulation.outputEncapsulationObject;

/**
 * @author XinYi Song
 */
@RestController
public class WorkHourRecordController {

    @Resource
    private WorkHourRecordService workHourRecordService;

    @Resource
    private SysUserDao sysUserDao;

    @Resource
    private WorkHourRecordDao workHourRecordDao;

    /**
     * 提交工作记录
     * @param workRecordQo
     * @param token
     * @return
     */
    @PostMapping("/submitWorkContent")
    @PreAuthorize("hasAnyAuthority('auth_administor','auth_general_user')")
    public String submitWorkContent(@RequestBody WorkRecordQo workRecordQo, @RequestHeader(value="Authorization") String token){
        return workHourRecordService.submitWorkContent(workRecordQo, token);
    }

    /**
     * 查询我参与的
     * @param token
     * @return
     */
    @GetMapping("/selectMemberWorkRecord")
    @PreAuthorize("hasAnyAuthority('auth_administor','auth_general_user')")
    public String selectMemberWorkRecord(@RequestHeader(value="Authorization") String token){
        // 获取区域信息
        Locale locale = LocaleContextHolder.getLocale();
        // 验证token
        String tokenUserName = TokenUtil.getTokenUserName(token);
        SysUserEntity sysUserEntity = sysUserDao.selectByUserName(tokenUserName);
        if(sysUserEntity == null){
            return outputEncapsulationObject(PromptMessageEnum.USER_LOGIN_ERROR,"您还没有注册登录，请先注册登录",locale);
        }
        List<MemberWorkRecordVo> memberWorkRecordVos = workHourRecordService.selectMemberWorkRecord(sysUserEntity.getId());
        if(memberWorkRecordVos.size() == 0){
            return outputEncapsulationObject(PromptMessageEnum.DATA_NONE,"暂时还没有任何数据",locale);
        }
        return outputEncapsulationObject(PromptMessageEnum.SUCCESS,memberWorkRecordVos,locale);
    }

    /**
     * 查询我管理的项目
     * @param token
     * @return
     */
    @GetMapping("/selectManagementProject")
    @PreAuthorize("hasAnyAuthority('auth_administor','auth_general_user')")
    public String selectManagementProject(@RequestHeader(value="Authorization") String token) throws Exception {
        // 获取区域信息
        Locale locale = LocaleContextHolder.getLocale();
        // 验证token
        String tokenUserName = TokenUtil.getTokenUserName(token);
        SysUserEntity sysUserEntity = sysUserDao.selectByUserName(tokenUserName);
        if(sysUserEntity == null){
            return outputEncapsulationObject(PromptMessageEnum.USER_LOGIN_ERROR,"您还没有注册登录，请先注册登录",locale);
        }
        List<Map> managementProjectVos = workHourRecordService.selectManagementProject(sysUserEntity.getId());
        if(managementProjectVos == null || managementProjectVos.size() == 0){
            return outputEncapsulationObject(PromptMessageEnum.DATA_NONE,"暂时还没有任何数据",locale);
        }
        return outputEncapsulationObject(PromptMessageEnum.SUCCESS,managementProjectVos,locale);
    }

    /**
     * 查询我参与的当前项目的提交的工时记录
     * @param map
     * @param token
     * @return
     */
    @PostMapping("/findRecord")
    @PreAuthorize("hasAnyAuthority('auth_administor','auth_general_user')")
    public String findRecord(@RequestBody Map map,@RequestHeader(value="Authorization") String token){
        // 获取区域信息
        Locale locale = LocaleContextHolder.getLocale();
        String projectNumber = (String) map.get("projectNumber");
        String startTime = (String) map.get("startTime");
        String endTime = (String) map.get("endTime");
        // 验证token
        String tokenUserName = TokenUtil.getTokenUserName(token);
        SysUserEntity sysUserEntity = sysUserDao.selectByUserName(tokenUserName);
        List<WorkHourRecord> workHourRecords = workHourRecordService.findRecord(projectNumber, sysUserEntity.getId(),startTime,endTime);
        if(workHourRecords == null || workHourRecords.size() == 0){
            return outputEncapsulationObject(PromptMessageEnum.DATA_NONE,"暂时还没有该项目的工时记录",locale);
        }
        return outputEncapsulationObject(PromptMessageEnum.SUCCESS,workHourRecords,locale);
    }

    /**
     * 修改工时记录
     * @param map
     * @return
     */
    @PostMapping("/updateRecord")
    @PreAuthorize("hasAnyAuthority('auth_administor','auth_general_user')")
    public String updateRecord(@RequestBody Map map,@RequestHeader(value="Authorization") String token){
        String workContent = (String) map.get("workContent");
        Integer id = (Integer) map.get("id");
        // 获取区域信息
        Locale locale = LocaleContextHolder.getLocale();
        String tokenUserName = TokenUtil.getTokenUserName(token);
        SysUserEntity sysUserEntity = sysUserDao.selectByUserName(tokenUserName);
        if(sysUserEntity == null){
            return outputEncapsulationObject(PromptMessageEnum.USER_LOGIN_ERROR,"您还没有注册登录，请先注册登录",locale);
        }
        workHourRecordService.updateRecord(workContent,id);
        return outputEncapsulationObject(PromptMessageEnum.SUCCESS,"修改成功",locale);
    }

    /**
     * 删除记录
     * @param map
     * @return
     */
    @PostMapping("/deleteRecord")
    @PreAuthorize("hasAnyAuthority('auth_administor','auth_general_user')")
    public String deleteRecord(@RequestBody Map map,@RequestHeader(value="Authorization") String token){
        Integer id = (Integer) map.get("id");
        // 获取区域信息
        Locale locale = LocaleContextHolder.getLocale();
        String tokenUserName = TokenUtil.getTokenUserName(token);
        SysUserEntity sysUserEntity = sysUserDao.selectByUserName(tokenUserName);
        if(sysUserEntity == null){
            return outputEncapsulationObject(PromptMessageEnum.USER_LOGIN_ERROR,"您还没有注册登录，请先注册登录",locale);
        }
        workHourRecordService.deleteById(id);
        return outputEncapsulationObject(PromptMessageEnum.SUCCESS,"删除成功",locale);
    }

    /**
     * 导出excel表
     * @param projectNumber
     * @param response
     * @throws IOException
     */
    @GetMapping("/excelOutWork")
    public void selectWork(@RequestParam("projectNumber") String projectNumber, HttpServletResponse response) throws IOException{
        workHourRecordService.selectWork(projectNumber,response);
    }

    /**
     * 查询该项目的成员和工时
     * @return
     */
    @GetMapping("/selectMemberAndWorkHour")
    public String selectMemberAndWorkHour(@RequestParam("projectNumber") String projectNumber){
        Locale locale = LocaleContextHolder.getLocale();
        List<OutMemberVo> outMemberVos = workHourRecordDao.selectMember(projectNumber);
        return outputEncapsulationObject(PromptMessageEnum.SUCCESS,outMemberVos,locale);
    }

    /**
     * 管理员查看成员的项目信息和工时
     * @param userId
     * @param token
     * @return
     */
    @GetMapping("/selectProjectNameAndHour")
    @PreAuthorize("hasAnyAuthority('auth_administor')")
    public String selectProjectNameAndHour(@RequestParam("userId") Integer userId,@RequestHeader(value="Authorization") String token){
        Locale locale = LocaleContextHolder.getLocale();
        List<Map<String, Object>> maps = workHourRecordDao.selectProjectNameAndHour(userId);
        if(maps == null || maps.size() == 0){
            return outputEncapsulationObject(PromptMessageEnum.DATA_NONE,"暂时没有该成员的项目信息",locale);
        }
        return outputEncapsulationObject(PromptMessageEnum.SUCCESS,maps,locale);
    }

    /**
     * 查询负责人和成员的项目起止时间，时长用于甘特图
     * @param projectNumber
     * @param token
     * @return
     */
    @GetMapping("/selectProjectByGanttChart")
    @PreAuthorize("hasAnyAuthority('auth_administor','auth_general_user')")
    public String selectProjectByGanttChart(@RequestParam("projectNumber") String projectNumber,@RequestHeader(value="Authorization") String token){
        Locale locale = LocaleContextHolder.getLocale();
        List<Map<String, Object>> maps1 = workHourRecordDao.selectWorkHourMember(projectNumber);
        if(maps1.size() == 0){
            return outputEncapsulationObject(PromptMessageEnum.DATA_NONE,"暂时还没有任何数据",locale);
        }
        return outputEncapsulationObject(PromptMessageEnum.SUCCESS,maps1,locale);
    }
}
