package com.xkrs.dao;

import com.xkrs.model.entity.ProjectOverview;
import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.data.jpa.repository.JpaSpecificationExecutor;
import org.springframework.data.jpa.repository.Modifying;
import org.springframework.data.jpa.repository.Query;
import org.springframework.stereotype.Component;

/**
 * @author XinYi Song
 */
@Component
public interface ProjectOverviewDao extends JpaRepository<ProjectOverview,Long>, JpaSpecificationExecutor<ProjectOverview> {

    /**
     * 修改项目的状态
     * @param projectStatus
     * @param projectNumber
     */
    @Query(value = "update project_overview set project_status = ?1 where project_number = ?2",nativeQuery = true)
    @Modifying(clearAutomatically=true)
    void updateProjectStatus(String projectStatus,String projectNumber);

    /**
     * 修改项目的授权状态
     * @param authorizationType
     * @param projectNumber
     */
    @Query(value = "update project_overview set authorization_type = ?1 where project_number = ?2",nativeQuery = true)
    @Modifying(clearAutomatically=true)
    void updateAuthorizationType(String authorizationType,String projectNumber);

    /**
     * 根据项目编号查询项目的信息
     * @param projectNumber
     * @return
     */
    ProjectOverview findByProjectNumber(String projectNumber);

    /**
     * 根据项目名称查询项目信息
     * @param projectName
     * @return
     */
    ProjectOverview findByProjectName(String projectName);

    /**
     * 根据项目编号删除项目
     * @param projectNumber
     */
    @Modifying(clearAutomatically=true)
    void deleteByProjectNumber(String projectNumber);
}
