package com.xkrs.dao;

import com.xkrs.model.entity.WorkHourRecord;
import com.xkrs.model.vo.MemberWorkRecordVo;
import com.xkrs.model.vo.OutExcelVo;
import com.xkrs.model.vo.OutMemberVo;
import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.data.jpa.repository.JpaSpecificationExecutor;
import org.springframework.data.jpa.repository.Modifying;
import org.springframework.data.jpa.repository.Query;
import org.springframework.data.repository.query.Param;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.Map;

/**
 * @author XinYi Song
 */
@Component
public interface WorkHourRecordDao extends JpaRepository<WorkHourRecord,Long>, JpaSpecificationExecutor<WorkHourRecord> {

    /**
     * 查询我参与的
     * @param userId
     * @return
     */
    @Query(value = "select new com.xkrs.model.vo.MemberWorkRecordVo " +
            "(po.projectNumber,po.projectName,po.projectStage,po.projectStatus,sum(whr.workHour),mp.joinTime) " +
            "from WorkHourRecord whr,ProjectOverview po,MemberProject mp " +
            "where po.projectNumber = mp.memberProjectNumber and whr.projectNumber = mp.memberProjectNumber " +
            "and whr.userId = mp.memberId and mp.applicationType = '1' and mp.memberId = :userId " +
            "group by po.projectNumber,po.projectName,po.projectStage,po.projectStatus,mp.joinTime")
    List<MemberWorkRecordVo> selectMemberWorkRecord(@Param("userId") Integer userId);

    /**
     * 负责人管理的项目（参与的）
     * @param userId
     * @return
     */
    /*@Query(value = "select new com.xkrs.model.vo.MemberWorkRecordVo " +
            "(po.projectNumber,po.projectName,po.projectStage,po.projectStatus,sum(whr.workHour)) " +
            "from WorkHourRecord whr,ProjectOverview po,ResponsibleProject rp " +
            "where po.projectNumber = rp.responsibleProjectNumber and whr.projectNumber = rp.responsibleProjectNumber " +
            "and whr.userId = rp.principalId and rp.auditType = '1' and rp.principalId = :userId " +
            "group by po.projectNumber,po.projectName,po.projectStage,po.projectStatus")
    List<MemberWorkRecordVo> selectMemberAndParWorkRecord(@Param("userId") Integer userId);*/


    /**
     * 查询我管理的项目
     * @param userId
     * @return
     */
    @Query(value = "select po.project_number projectNumber,po.project_name projectName,po.project_stage projectStage,po.project_status projectStatus,(select count(mp.*) from member_project mp where mp.member_project_number = po.project_number and mp.application_type = '1') as peopleNum,sum(whr.work_hour) as workHour " +
            "from work_hour_record whr,project_overview po,responsible_project rp " +
            "where po.project_number = rp.responsible_project_number " +
            "and whr.project_number = rp.responsible_project_number " +
            "and rp.audit_type = '1' " +
            "and rp.principal_id = :userId " +
            "group by po.project_number,po.project_name,po.project_stage,po.project_status",nativeQuery = true)
    List<Map> selectManagementProject(@Param("userId") Integer userId);

    /**
     * 查询我参与的当前项目的提交的工时记录
     * @param projectNumber
     * @param userId
     * @return
     */
    @Query(value = "select * from work_hour_record where color != '1' and project_number = :projectNumber and user_id = :userId",nativeQuery = true)
    List<WorkHourRecord> findRecord(@Param("projectNumber") String projectNumber,@Param("userId") Integer userId);

    /**
     * 修改工时记录
     * @param workContent
     * @param
     * @param id
     */
    @Query(value = "update work_hour_record set work_content = ?1 where id = ?2",nativeQuery = true)
    @Modifying(clearAutomatically=true)
    void updateRecord(String workContent,Integer id);

    /**
     * 删除记录
     * @param id
     */
    @Modifying(clearAutomatically=true)
    @Query(value = "delete from work_hour_record where id = ?",nativeQuery = true)
    void deleteById(Integer id);

    /**
     * 用于导出excel表格
     * @param projectNumber
     * @return
     */
    @Query(value = "select new com.xkrs.model.vo.OutExcelVo (po.projectNumber,po.projectName,po.projectStage,sum(whr.workHour)) " +
            "from WorkHourRecord whr,ProjectOverview po where whr.projectNumber = po.projectNumber " +
            "and po.projectNumber = :projectNumber group by po.projectNumber,po.projectName,po.projectStage")
    OutExcelVo selectWork(@Param("projectNumber") String projectNumber);

    /**
     * 查询人员
     * @param projectNumber
     * @return
     */
    @Query(value = "select new com.xkrs.model.vo.OutMemberVo (su.reallyName,su.userDepartment,sum(whr.workHour)) from WorkHourRecord whr,SysUserEntity su where su.id = whr.userId and whr.projectNumber = :projectNumber group by su.reallyName,su.userDepartment")
    List<OutMemberVo> selectMember(@Param("projectNumber") String projectNumber);

    /**
     * 查询个人参加的项目和工时
     * @param userId
     * @return
     */
    @Query(value = "select po.project_name projectname,po.project_number projectnumber,sum(whr.work_hour) as workhour " +
            "from project_overview po,work_hour_record whr " +
            "where po.project_number = whr.project_number and whr.user_id = :userId " +
            "GROUP BY po.project_name,po.project_number",nativeQuery = true)
    List<Map<String,Object>> selectProjectNameAndHour(@Param("userId") Integer userId);

    /**
     * 查询每个负责人在该项目中的时长，起止时间用于甘特图
     * @param projectNumber
     * @return
     */
    @Query(value = "select su.really_name reallyname,rp.start_time starttime,sum(whr.work_hour) as workhour," +
            "(select submit_time from work_hour_record where user_id = rp.principal_id " +
            "ORDER BY submit_time DESC limit 1) as lasttime from sys_user su,responsible_project rp,work_hour_record whr " +
            "where whr.project_number = rp.responsible_project_number AND whr.user_id = rp.principal_id " +
            "and su.id = rp.principal_id AND rp.audit_type = '1' AND rp.responsible_project_number = :projectNumber " +
            "GROUP BY su.really_name,rp.start_time,lasttime",nativeQuery = true)
    List<Map<String,Object>> selectWorkHourPar(@Param("projectNumber") String projectNumber);

    /**
     * 查询每个成员在该项目中的时长和起止时间用于甘特图
     * @param projectNumber
     * @return
     */
    @Query(value = "select su.really_name reallyname,mp.join_time starttime,sum(whr.work_hour) as workhour," +
            "(select submit_time from work_hour_record where user_id = mp.member_id " +
            "ORDER BY submit_time DESC limit 1) as lasttime from sys_user su,member_project mp,work_hour_record whr " +
            "where whr.project_number = mp.member_project_number AND whr.user_id = mp.member_id AND su.id = mp.member_id AND mp.application_type = '1' AND mp.member_project_number = :projectNumber GROUP BY su.really_name,mp.join_time,lasttime",nativeQuery = true)
    List<Map<String,Object>> selectWorkHourMember(@Param("projectNumber") String projectNumber);
}
