package com.xkrs.model.entity;

import javax.persistence.*;
import java.io.Serializable;

/**
 * @author XinYi Song
 * 项目总览
 */
@Entity
@Table(name="project_overview")
public class ProjectOverview implements Serializable {
    /**
     * 指定主键，建立自增序列，主键值取自序列
     */
    @Id
    @GeneratedValue(strategy = GenerationType.SEQUENCE, generator = "project_overview_seq_gen")
    @SequenceGenerator(name = "project_overview_seq_gen", sequenceName = "project_overview_id_seq",allocationSize = 1)
    private Integer id;

    /**
     * 项目编号
     */
    @Column(length = 32,columnDefinition = "varchar(32)")
    private String projectNumber;

    /**
     * 项目名称
     */
    private String projectName;

    /**
     * 项目类型（遥感  视觉）
     */
    @Column(length = 32,columnDefinition = "varchar(32)")
    private String projectType;

    /**
     * 项目阶段（A-前期交流；B-实施中；C-已完结；D-未赢取）
     */
    @Column(length = 32,columnDefinition = "varchar(32)")
    private String projectStage;

    /**
     * 甲方
     */
    @Column(length = 85, columnDefinition = "varchar(85)")
    private String party;

    /**
     * 市场
     */
    @Column(length = 32,columnDefinition = "varchar(32)")
    private String projectMarket;

    /**
     * 售前
     */
    @Column(length = 32,columnDefinition = "varchar(32)")
    private String projectSale;

    /**
     * 项目经理
     */
    @Column(length = 32,columnDefinition = "varchar(32)")
    private String projectManager;

    /**
     * 项目填报状态  0 未开始 1 填报中
     */
    @Column(length = 32,columnDefinition = "varchar(32)")
    private String projectStatus;

    /**
     * 授权状态值    0未授权给任何人    1已授权给他人
     */
    @Column(length = 32,columnDefinition = "varchar(32)")
    private String authorizationType;

    public ProjectOverview() {
    }

    public ProjectOverview(Integer id, String projectNumber, String projectName, String projectType, String projectStage, String party, String projectMarket, String projectSale, String projectManager, String projectStatus, String authorizationType) {
        this.id = id;
        this.projectNumber = projectNumber;
        this.projectName = projectName;
        this.projectType = projectType;
        this.projectStage = projectStage;
        this.party = party;
        this.projectMarket = projectMarket;
        this.projectSale = projectSale;
        this.projectManager = projectManager;
        this.projectStatus = projectStatus;
        this.authorizationType = authorizationType;
    }

    public Integer getId() {
        return id;
    }

    public void setId(Integer id) {
        this.id = id;
    }

    public String getProjectNumber() {
        return projectNumber;
    }

    public void setProjectNumber(String projectNumber) {
        this.projectNumber = projectNumber;
    }

    public String getProjectName() {
        return projectName;
    }

    public void setProjectName(String projectName) {
        this.projectName = projectName;
    }

    public String getProjectType() {
        return projectType;
    }

    public void setProjectType(String projectType) {
        this.projectType = projectType;
    }

    public String getProjectStage() {
        return projectStage;
    }

    public void setProjectStage(String projectStage) {
        this.projectStage = projectStage;
    }

    public String getParty() {
        return party;
    }

    public void setParty(String party) {
        this.party = party;
    }

    public String getProjectMarket() {
        return projectMarket;
    }

    public void setProjectMarket(String projectMarket) {
        this.projectMarket = projectMarket;
    }

    public String getProjectSale() {
        return projectSale;
    }

    public void setProjectSale(String projectSale) {
        this.projectSale = projectSale;
    }

    public String getProjectManager() {
        return projectManager;
    }

    public void setProjectManager(String projectManager) {
        this.projectManager = projectManager;
    }

    public String getProjectStatus() {
        return projectStatus;
    }

    public void setProjectStatus(String projectStatus) {
        this.projectStatus = projectStatus;
    }

    public String getAuthorizationType() {
        return authorizationType;
    }

    public void setAuthorizationType(String authorizationType) {
        this.authorizationType = authorizationType;
    }

    @Override
    public String toString() {
        return "ProjectOverview{" +
                "id=" + id +
                ", projectNumber='" + projectNumber + '\'' +
                ", projectName='" + projectName + '\'' +
                ", projectType='" + projectType + '\'' +
                ", projectStage='" + projectStage + '\'' +
                ", party='" + party + '\'' +
                ", projectMarket='" + projectMarket + '\'' +
                ", projectSale='" + projectSale + '\'' +
                ", projectManager='" + projectManager + '\'' +
                ", projectStatus='" + projectStatus + '\'' +
                ", authorizationType='" + authorizationType + '\'' +
                '}';
    }
}
