package com.xkrs.model.entity;

import javax.persistence.*;
import java.io.Serializable;
import java.time.LocalDateTime;


/**
 * SysUser 表实体类
 * @author tajochen
 */
@Entity
@Table(name="sys_user")
public class SysUserEntity implements Serializable {

    /**
     * 指定主键，建立自增序列，主键值取自序列
     */
    @Id
    @GeneratedValue(strategy = GenerationType.SEQUENCE, generator = "sys_user_seq_gen")
    @SequenceGenerator(name = "sys_user_seq_gen", sequenceName = "sys_user_id_seq",allocationSize = 1)
    private Integer id;

    /**
     * 用户名
     */
    @Column(length = 16, nullable = false, unique = true, columnDefinition = "varchar(16)")
    private String userName;

    /**
     * 用户真实姓名
     */
    @Column(length = 32, columnDefinition = "varchar(32)")
    private String reallyName;

    /**
     * 密码
     */
    @Column(length = 64, nullable = false, columnDefinition = "varchar(64)")
    private String password;

    @Column(length = 32, nullable = false, columnDefinition = "varchar(32)")
    private String salt;

    @Column(length = 16, unique = true, columnDefinition = "varchar(16)")
    private String telephone;

    /**
     * 所在部门
     */
    @Column(length = 32, columnDefinition = "varchar(32)")
    private String userDepartment;

    @Column(columnDefinition = "varchar(192)")
    private String signature;

    @Column(nullable = false)
    private Integer activeFlag;

    @Column(nullable = false,columnDefinition = "smallint")
    private Integer statusCode;

    @Column(nullable = false)
    private LocalDateTime addTime;

    private LocalDateTime lastEntryTime;

    @Column(nullable = false)
    private Integer deleteFlag;

    @Column(columnDefinition = "varchar(64)")
    private String lastEntryIp;

    @Column(columnDefinition = "varchar(88)")
    private String openId;

    /**
     * 微信扫码登录的openId
     */
    @Column(columnDefinition = "varchar(88)")
    private String weChatOpenId;

    public Integer getId() {
        return id;
    }

    public void setId(Integer id) {
        this.id = id;
    }

    public String getUserName() {
        return userName;
    }

    public void setUserName(String userName) {
        this.userName = userName;
    }

    public String getReallyName() {
        return reallyName;
    }

    public void setReallyName(String reallyName) {
        this.reallyName = reallyName;
    }

    public String getPassword() {
        return password;
    }

    public void setPassword(String password) {
        this.password = password;
    }

    public String getSalt() {
        return salt;
    }

    public void setSalt(String salt) {
        this.salt = salt;
    }

    public String getTelephone() {
        return telephone;
    }

    public void setTelephone(String telephone) {
        this.telephone = telephone;
    }

    public String getUserDepartment() {
        return userDepartment;
    }

    public void setUserDepartment(String userDepartment) {
        this.userDepartment = userDepartment;
    }

    public String getSignature() {
        return signature;
    }

    public void setSignature(String signature) {
        this.signature = signature;
    }

    public Integer getActiveFlag() {
        return activeFlag;
    }

    public void setActiveFlag(Integer activeFlag) {
        this.activeFlag = activeFlag;
    }

    public Integer getStatusCode() {
        return statusCode;
    }

    public void setStatusCode(Integer statusCode) {
        this.statusCode = statusCode;
    }

    public LocalDateTime getAddTime() {
        return addTime;
    }

    public void setAddTime(LocalDateTime addTime) {
        this.addTime = addTime;
    }

    public LocalDateTime getLastEntryTime() {
        return lastEntryTime;
    }

    public void setLastEntryTime(LocalDateTime lastEntryTime) {
        this.lastEntryTime = lastEntryTime;
    }

    public Integer getDeleteFlag() {
        return deleteFlag;
    }

    public void setDeleteFlag(Integer deleteFlag) {
        this.deleteFlag = deleteFlag;
    }

    public String getLastEntryIp() {
        return lastEntryIp;
    }

    public void setLastEntryIp(String lastEntryIp) {
        this.lastEntryIp = lastEntryIp;
    }

    public String getOpenId() {
        return openId;
    }

    public void setOpenId(String openId) {
        this.openId = openId;
    }

    public String getWeChatOpenId() {
        return weChatOpenId;
    }

    public void setWeChatOpenId(String weChatOpenId) {
        this.weChatOpenId = weChatOpenId;
    }

    @Override
    public String toString() {
        return "SysUserEntity{" +
                "id=" + id +
                ", userName='" + userName + '\'' +
                ", reallyName='" + reallyName + '\'' +
                ", password='" + password + '\'' +
                ", salt='" + salt + '\'' +
                ", telephone='" + telephone + '\'' +
                ", userDepartment='" + userDepartment + '\'' +
                ", signature='" + signature + '\'' +
                ", activeFlag=" + activeFlag +
                ", statusCode=" + statusCode +
                ", addTime=" + addTime +
                ", lastEntryTime=" + lastEntryTime +
                ", deleteFlag=" + deleteFlag +
                ", lastEntryIp='" + lastEntryIp + '\'' +
                ", openId='" + openId + '\'' +
                ", weChatOpenId='" + weChatOpenId + '\'' +
                '}';
    }
}
