package com.xkrs.service.impl;

import com.xkrs.dao.ProjectOverviewDao;
import com.xkrs.model.entity.ProjectOverview;
import com.xkrs.model.qo.ProjectQo;
import com.xkrs.model.qo.ProjectUpdateQo;
import com.xkrs.service.ProjectOverviewService;
import com.xkrs.utils.ExcelImportUtil;
import com.xkrs.utils.Query;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.transaction.Transactional;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * @author XinYi Song
 */
@Service
public class ProjectOverviewServiceImpl implements ProjectOverviewService {

    @Resource
    private ProjectOverviewDao projectOverviewDao;

    @Resource
    private Query query;

    /**
     * 导入遥感项目excel表
     * @param file
     * @throws Exception
     */
    @Transactional(rollbackOn = Exception.class)
    @Override
    public void importRsProjectExcel(MultipartFile file) throws Exception {
        //调用封装好的工具
        ExcelImportUtil importUtil = new ExcelImportUtil(file);
        //调用导入的方法，获取sheet表的内容
        List<Map<String, String>> maps = importUtil.readExcelContent();
        //导入订单详情表
        List<ProjectOverview> orderDetails = maps.stream().filter(Objects::nonNull).map(map -> {
            ProjectOverview projectOverview = new ProjectOverview();
            projectOverview.setProjectNumber(map.get("项目编号").substring(0,10));
            projectOverview.setProjectName(map.get("RS-项目名称"));
            projectOverview.setProjectType("遥感");
            projectOverview.setProjectStage(map.get("阶段"));
            projectOverview.setParty(map.get("甲方"));
            projectOverview.setProjectMarket(map.get("市场人员"));
            projectOverview.setProjectSale(map.get("售前"));
            projectOverview.setProjectManager(map.get("项目经理"));
            projectOverview.setProjectStatus("0");
            projectOverview.setAuthorizationType("0");
            return projectOverview;
        }).collect(Collectors.toList());
        //批量添加到订单详情
        projectOverviewDao.saveAll(orderDetails);
    }

    /**
     * 导入视觉项目的excel表
     * @param file
     * @throws Exception
     */
    @Transactional(rollbackOn = Exception.class)
    @Override
    public void importCvProjectExcel(MultipartFile file) throws Exception {
        //调用封装好的工具
        ExcelImportUtil importUtil = new ExcelImportUtil(file);
        //调用导入的方法，获取sheet表的内容
        List<Map<String, String>> maps = importUtil.readExcelContent();
        //导入订单详情表
        List<ProjectOverview> orderDetails = maps.stream().filter(Objects::nonNull).map(map -> {
            ProjectOverview projectOverview = new ProjectOverview();
            projectOverview.setProjectNumber(map.get("CV-项目编号").substring(0,10));
            projectOverview.setProjectName(map.get("CV-项目名称"));
            projectOverview.setProjectType("视觉");
            projectOverview.setParty(map.get("甲方"));
            projectOverview.setProjectStage(map.get("阶段"));
            projectOverview.setProjectMarket(map.get("市场人员"));
            projectOverview.setProjectSale(map.get("售前"));
            projectOverview.setProjectManager(map.get("项目经理"));
            projectOverview.setProjectStatus("0");
            projectOverview.setAuthorizationType("0");
            return projectOverview;
        }).collect(Collectors.toList());
        //批量添加到订单详情
        projectOverviewDao.saveAll(orderDetails);
    }

    /**
     * 动态多条件查询项目信息
     * @param projectNumber
     * @param projectName
     * @return
     */
    @Override
    public List<ProjectOverview> selectProjectByDynamic(String projectNumber, String projectName) {
        List<ProjectOverview> projectOverviews = query.selectProjectByDynamic(projectNumber, projectName);
        return projectOverviews;
    }

    /**
     * 新增项目
     * @param projectQo
     * @return
     */
    @Override
    public ProjectOverview insertProjectOverview(ProjectQo projectQo) {
        ProjectOverview projectOverview = new ProjectOverview();
        projectOverview.setProjectNumber(projectQo.getProjectNumber());
        projectOverview.setProjectName(projectQo.getProjectName());
        projectOverview.setProjectType(projectQo.getProjectType());
        projectOverview.setParty(projectQo.getParty());
        projectOverview.setProjectStage(projectQo.getProjectStage());
        projectOverview.setProjectMarket(projectQo.getProjectMarket());
        projectOverview.setProjectSale(projectQo.getProjectSale());
        projectOverview.setProjectManager(projectQo.getProjectManager());
        if("QT".equals(projectQo.getProjectNumber().substring(0,2))){
            projectOverview.setProjectStatus("1");
            return projectOverviewDao.save(projectOverview);
        }
        projectOverview.setProjectStatus("0");
        projectOverview.setAuthorizationType("0");
        return projectOverviewDao.save(projectOverview);
    }

    /**
     * 修改项目
     * @param projectUpdateQo
     */
    @Override
    public void updateProjectOverview(ProjectUpdateQo projectUpdateQo) {
        ProjectOverview projectOverview = new ProjectOverview();
        projectOverview.setId(projectUpdateQo.getId());
        projectOverview.setProjectNumber(projectUpdateQo.getProjectNumber());
        projectOverview.setProjectName(projectUpdateQo.getProjectName());
        projectOverview.setProjectType(projectUpdateQo.getProjectType());
        projectOverview.setParty(projectUpdateQo.getParty());
        projectOverview.setProjectStage(projectUpdateQo.getProjectStage());
        projectOverview.setProjectMarket(projectUpdateQo.getProjectMarket());
        projectOverview.setProjectSale(projectUpdateQo.getProjectSale());
        projectOverview.setProjectManager(projectUpdateQo.getProjectManager());
        projectOverview.setProjectStatus(projectUpdateQo.getProjectStatus());
        projectOverview.setAuthorizationType(projectUpdateQo.getAuthorizationType());
        projectOverviewDao.save(projectOverview);

    }

    /**
     * 根据项目编号删除项目
     * @param projectNumber
     */
    @Transactional(rollbackOn = Exception.class)
    @Override
    public void deleteByProjectNumber(String projectNumber) {
        projectOverviewDao.deleteByProjectNumber(projectNumber);
    }
}
