package com.xkrs.service.impl;

import com.xkrs.common.encapsulation.PromptMessageEnum;
import com.xkrs.common.tool.TokenUtil;
import com.xkrs.dao.SysUserDao;
import com.xkrs.dao.WorkHourRecordDao;
import com.xkrs.model.entity.SysUserEntity;
import com.xkrs.model.entity.WorkHourRecord;
import com.xkrs.model.qo.WorkRecordQo;
import com.xkrs.model.vo.MemberWorkRecordVo;
import com.xkrs.model.vo.OutExcelVo;
import com.xkrs.model.vo.OutMemberVo;
import com.xkrs.service.WorkHourRecordService;
import com.xkrs.utils.DateTimeUtil;
import com.xkrs.utils.Query;
import org.apache.poi.hssf.usermodel.*;
import org.apache.poi.ss.usermodel.*;
import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import javax.transaction.Transactional;
import java.io.IOException;
import java.time.LocalDateTime;
import java.util.List;
import java.util.Locale;
import java.util.Map;

import static com.xkrs.common.encapsulation.OutputEncapsulation.outputEncapsulationObject;

/**
 * @author XinYi Song
 */
@Service
public class WorkHourRecordServiceImpl implements WorkHourRecordService {

    @Resource
    private WorkHourRecordDao workHourRecordDao;

    @Resource
    private SysUserDao sysUserDao;

    @Resource
    private Query query;

    /**
     * 提交工时记录
     * @param workRecordQo
     * @param token
     * @return
     */
    @Override
    public String submitWorkContent(WorkRecordQo workRecordQo, String token) {
        // 获取区域信息
        Locale locale = LocaleContextHolder.getLocale();
        // 验证token
        String tokenUserName = TokenUtil.getTokenUserName(token);
        SysUserEntity sysUserEntity = sysUserDao.selectByUserName(tokenUserName);
        if(sysUserEntity == null){
            return outputEncapsulationObject(PromptMessageEnum.USER_LOGIN_ERROR,"您还没有注册登录，请先注册登录",locale);
        }
        if(workRecordQo.getWorkHour() > 24){
            return outputEncapsulationObject(PromptMessageEnum.DATA_WRONG,"工时不能超过24小时，请重新填写",locale);
        }
        WorkHourRecord workHourRecord = new WorkHourRecord();
        workHourRecord.setWorkContent(workRecordQo.getWorkContent());
        workHourRecord.setWorkHour(workRecordQo.getWorkHour());
        workHourRecord.setColor(workRecordQo.getColor());
        workHourRecord.setUserId(sysUserEntity.getId());
        workHourRecord.setProjectNumber(workRecordQo.getProjectNumber());
        workHourRecord.setSubmitTime(workRecordQo.getSubmitTime());
        WorkHourRecord save = workHourRecordDao.save(workHourRecord);
        return outputEncapsulationObject(PromptMessageEnum.SUCCESS,"提交成功",locale);
    }

    /**
     * 查询我参与的
     * @param userId
     * @return
     */
    @Override
    public List<MemberWorkRecordVo> selectMemberWorkRecord(Integer userId) {
        return workHourRecordDao.selectMemberWorkRecord(userId);
    }

    /**
     * 查询
     * @param userId
     * @return
     */
    /*@Override
    public List<MemberWorkRecordVo> selectMemberAndParWorkRecord(Integer userId) {
        return workHourRecordDao.selectMemberAndParWorkRecord(userId);
    }*/

    /**
     * 查询我管理的项目
     * @param userId
     * @return
     */
    @Override
    public List<Map> selectManagementProject(Integer userId) {
        return workHourRecordDao.selectManagementProject(userId);
    }

    /**
     * 查询我参与的当前项目的提交的工时记录
     * @param projectNumber
     * @param userId
     * @param startTime
     * @param endTime
     * @return
     */
    @Override
    public List<WorkHourRecord> findRecord(String projectNumber, Integer userId, String startTime, String endTime) {
        return query.selectWorkRecord(projectNumber,userId,startTime,endTime);
    }

    /**
     * 修改工时记录
     * @param workContent
     * @param id
     */
    @Transactional(rollbackOn = Exception.class)
    @Override
    public void updateRecord(String workContent, Integer id) {
        //String time = DateTimeUtil.dateTimeToString(LocalDateTime.now());
        workHourRecordDao.updateRecord(workContent,id);
    }

    /**
     * 删除记录
     * @param id
     */
    @Transactional(rollbackOn = Exception.class)
    @Override
    public void deleteById(Integer id) {
        workHourRecordDao.deleteById(id);
    }

    @Override
    public void selectWork(String projectNumber, HttpServletResponse response) throws IOException {
        OutExcelVo outExcelVo = workHourRecordDao.selectWork(projectNumber);
        //声明一个工作簿
        HSSFWorkbook workbook = new HSSFWorkbook();

        //生成一个表格，设置表格名称为"学生表"
        HSSFSheet sheet = workbook.createSheet("工时管理");
        //字号
        HSSFFont f  = workbook.createFont();
        f.setFontHeightInPoints((short) 16);

        // 设置列宽
        sheet.setColumnWidth(0,256*14+184);
        sheet.setColumnWidth(1,256*17+184);
        sheet.setColumnWidth(2,256*14+184);
        sheet.setColumnWidth(3,256*50+184);

        //创建第一行表头
        HSSFRow headrow = sheet.createRow(0);
        // 设置行高
        headrow.setHeight((short) (256*1+184));

        //创建一个单元格
        HSSFCell cell = headrow.createCell(0);
        CellStyle style=workbook.createCellStyle();
        style.setFont(f);
        //左右居中
        style.setAlignment(HorizontalAlignment.CENTER);
        //创建一个内容对象
        HSSFRichTextString text = new HSSFRichTextString("项目编号");
        //将内容对象的文字内容写入到单元格中
        cell.setCellValue(text);
        cell.setCellStyle(style);
        style.setFillForegroundColor(IndexedColors.GREY_25_PERCENT.getIndex());
        style.setFillPattern(FillPatternType.SOLID_FOREGROUND);
        //底边框加黑
        style.setBorderBottom(BorderStyle.THIN);
        //左边框加黑
        style.setBorderLeft(BorderStyle.THIN);
        // 有边框加黑
        style.setBorderRight(BorderStyle.THIN);
        //上边框加黑
        style.setBorderTop(BorderStyle.THIN);
        cell.setCellStyle(style);

        HSSFCell cell1 = headrow.createCell(1);
        CellStyle style1=workbook.createCellStyle();
        style1.setFont(f);
        //左右居中
        style1.setAlignment(HorizontalAlignment.CENTER);
        //创建一个内容对象
        HSSFRichTextString text1 = new HSSFRichTextString(outExcelVo.getProjectNumber());
        //将内容对象的文字内容写入到单元格中
        cell1.setCellValue(text1);
        style1.setBorderBottom(BorderStyle.THIN);
        //左边框加黑
        style1.setBorderLeft(BorderStyle.THIN);
        // 有边框加黑
        style1.setBorderRight(BorderStyle.THIN);
        //上边框加黑
        style1.setBorderTop(BorderStyle.THIN);
        cell1.setCellStyle(style1);

        HSSFCell cell2 = headrow.createCell(2);
        CellStyle style2=workbook.createCellStyle();
        style2.setFont(f);
        //左右居中
        style2.setAlignment(HorizontalAlignment.CENTER);
        //创建一个内容对象
        HSSFRichTextString text2 = new HSSFRichTextString("项目名称");
        //将内容对象的文字内容写入到单元格中
        cell2.setCellValue(text2);
        style2.setFillForegroundColor(IndexedColors.GREY_25_PERCENT.getIndex());
        style2.setFillPattern(FillPatternType.SOLID_FOREGROUND);
        style2.setBorderBottom(BorderStyle.THIN);
        //左边框加黑
        style2.setBorderLeft(BorderStyle.THIN);
        // 有边框加黑
        style2.setBorderRight(BorderStyle.THIN);
        //上边框加黑
        style2.setBorderTop(BorderStyle.THIN);
        cell2.setCellStyle(style2);

        HSSFCell cell3 = headrow.createCell(3);
        CellStyle style3=workbook.createCellStyle();
        style3.setFont(f);
        //左右居中
        style3.setAlignment(HorizontalAlignment.CENTER);
        //创建一个内容对象
        HSSFRichTextString text3 = new HSSFRichTextString(outExcelVo.getProjectName());
        //将内容对象的文字内容写入到单元格中
        cell3.setCellValue(text3);
        style3.setBorderBottom(BorderStyle.THIN);
        //左边框加黑
        style3.setBorderLeft(BorderStyle.THIN);
        // 有边框加黑
        style3.setBorderRight(BorderStyle.THIN);
        //上边框加黑
        style3.setBorderTop(BorderStyle.THIN);
        cell3.setCellStyle(style3);

        //创建第二行表头
        HSSFRow headrow1 = sheet.createRow(1);
        headrow1.setHeight((short) (256*1+184));
        //创建一个单元格
        HSSFCell cell11 = headrow1.createCell(0);
        CellStyle style4=workbook.createCellStyle();
        style4.setFont(f);
        //左右居中
        style4.setAlignment(HorizontalAlignment.CENTER);
        //创建一个内容对象
        HSSFRichTextString text11 = new HSSFRichTextString("总工时");
        //将内容对象的文字内容写入到单元格中
        cell11.setCellValue(text11);
        style4.setFillForegroundColor(IndexedColors.GREY_25_PERCENT.getIndex());
        style4.setFillPattern(FillPatternType.SOLID_FOREGROUND);
        style4.setBorderBottom(BorderStyle.THIN);
        //左边框加黑
        style4.setBorderLeft(BorderStyle.THIN);
        // 有边框加黑
        style4.setBorderRight(BorderStyle.THIN);
        //上边框加黑
        style4.setBorderTop(BorderStyle.THIN);
        cell11.setCellStyle(style4);

        HSSFCell cell12 = headrow1.createCell(1);
        CellStyle style5=workbook.createCellStyle();
        style5.setFont(f);
        //左右居中
        style5.setAlignment(HorizontalAlignment.CENTER);
        //创建一个内容对象
        HSSFRichTextString text12 = new HSSFRichTextString(String.valueOf(outExcelVo.getWorkHour()));
        //将内容对象的文字内容写入到单元格中
        cell12.setCellValue(text12);
        style5.setBorderBottom(BorderStyle.THIN);
        //左边框加黑
        style5.setBorderLeft(BorderStyle.THIN);
        // 有边框加黑
        style5.setBorderRight(BorderStyle.THIN);
        //上边框加黑
        style5.setBorderTop(BorderStyle.THIN);
        cell12.setCellStyle(style5);

        HSSFCell cell22 = headrow1.createCell(2);
        CellStyle style6=workbook.createCellStyle();
        style6.setFont(f);
        //左右居中
        style6.setAlignment(HorizontalAlignment.CENTER);
        //创建一个内容对象
        HSSFRichTextString text22 = new HSSFRichTextString("阶段");
        //将内容对象的文字内容写入到单元格中
        cell22.setCellValue(text22);
        style6.setFillForegroundColor(IndexedColors.GREY_25_PERCENT.getIndex());
        style6.setFillPattern(FillPatternType.SOLID_FOREGROUND);
        style6.setBorderBottom(BorderStyle.THIN);
        //左边框加黑
        style6.setBorderLeft(BorderStyle.THIN);
        // 有边框加黑
        style6.setBorderRight(BorderStyle.THIN);
        //上边框加黑
        style6.setBorderTop(BorderStyle.THIN);
        cell22.setCellStyle(style6);

        HSSFCell cell33 = headrow1.createCell(3);
        CellStyle style7=workbook.createCellStyle();
        style7.setFont(f);
        //左右居中
        style7.setAlignment(HorizontalAlignment.CENTER);
        // A-前期交流；B-实施中；C-已完结；D-未赢取
        if("A".equals(outExcelVo.getProjectStage())){
            //创建一个内容对象
            HSSFRichTextString text33 = new HSSFRichTextString("前期交流");
            //将内容对象的文字内容写入到单元格中
            cell33.setCellValue(text33);
        }else if("B".equals(outExcelVo.getProjectStage())){
            HSSFRichTextString text33 = new HSSFRichTextString("实施中");
            cell33.setCellValue(text33);
        }else if("C".equals(outExcelVo.getProjectStage())){
            HSSFRichTextString text33 = new HSSFRichTextString("已完结");
            cell33.setCellValue(text33);
        }else {
            HSSFRichTextString text33 = new HSSFRichTextString("未赢取");
            cell33.setCellValue(text33);
        }
        style7.setBorderBottom(BorderStyle.THIN);
        //左边框加黑
        style7.setBorderLeft(BorderStyle.THIN);
        // 有边框加黑
        style7.setBorderRight(BorderStyle.THIN);
        //上边框加黑
        style7.setBorderTop(BorderStyle.THIN);
        cell33.setCellStyle(style7);

        //创建第三行表头
        HSSFRow headrow2 = sheet.createRow(2);
        headrow2.setHeight((short) (256*1+184));
        //创建一个单元格
        HSSFCell cell21 = headrow2.createCell(0);
        CellStyle style8=workbook.createCellStyle();
        style8.setFont(f);
        //左右居中
        style8.setAlignment(HorizontalAlignment.CENTER);
        //创建一个内容对象
        HSSFRichTextString text21 = new HSSFRichTextString("人员");
        //将内容对象的文字内容写入到单元格中
        cell21.setCellValue(text21);
        style8.setFillForegroundColor(IndexedColors.GREY_25_PERCENT.getIndex());
        style8.setFillPattern(FillPatternType.SOLID_FOREGROUND);
        style8.setBorderBottom(BorderStyle.THIN);
        //左边框加黑
        style8.setBorderLeft(BorderStyle.THIN);
        // 有边框加黑
        style8.setBorderRight(BorderStyle.THIN);
        //上边框加黑
        style8.setBorderTop(BorderStyle.THIN);
        cell21.setCellStyle(style8);


        HSSFCell cell231 = headrow2.createCell(1);
        CellStyle style91=workbook.createCellStyle();
        style91.setFont(f);
        //左右居中
        style91.setAlignment(HorizontalAlignment.CENTER);
        //创建一个内容对象
        HSSFRichTextString text231 = new HSSFRichTextString("组织");
        //将内容对象的文字内容写入到单元格中
        cell231.setCellValue(text231);
        style91.setFillForegroundColor(IndexedColors.GREY_25_PERCENT.getIndex());
        style91.setFillPattern(FillPatternType.SOLID_FOREGROUND);
        style91.setBorderBottom(BorderStyle.THIN);
        //左边框加黑
        style91.setBorderLeft(BorderStyle.THIN);
        // 有边框加黑
        style91.setBorderRight(BorderStyle.THIN);
        //上边框加黑
        style91.setBorderTop(BorderStyle.THIN);
        cell231.setCellStyle(style91);


        HSSFCell cell23 = headrow2.createCell(2);
        CellStyle style9=workbook.createCellStyle();
        style9.setFont(f);
        //左右居中
        style9.setAlignment(HorizontalAlignment.CENTER);
        //创建一个内容对象
        HSSFRichTextString text23 = new HSSFRichTextString("工时");
        //将内容对象的文字内容写入到单元格中
        cell23.setCellValue(text23);
        style9.setFillForegroundColor(IndexedColors.GREY_25_PERCENT.getIndex());
        style9.setFillPattern(FillPatternType.SOLID_FOREGROUND);
        style9.setBorderBottom(BorderStyle.THIN);
        //左边框加黑
        style9.setBorderLeft(BorderStyle.THIN);
        // 有边框加黑
        style9.setBorderRight(BorderStyle.THIN);
        //上边框加黑
        style9.setBorderTop(BorderStyle.THIN);
        cell23.setCellStyle(style9);

        List<OutMemberVo> outMemberVos = workHourRecordDao.selectMember(projectNumber);
        int rowIndex = 3;
        for(int i = 0; i < outMemberVos.size(); i++){
            HSSFRow row1 = sheet.createRow(rowIndex++);
            row1.setHeight((short) (256*1+184));
            HSSFCell cell35 = row1.createCell(0);
            CellStyle style10=workbook.createCellStyle();
            style10.setFont(f);
            //左右居中
            style10.setAlignment(HorizontalAlignment.CENTER);
            HSSFRichTextString text35 = new HSSFRichTextString(outMemberVos.get(i).getReallyName());
            cell35.setCellValue(text35);
            cell35.setCellStyle(style10);

            HSSFCell cell351 = row1.createCell(1);
            HSSFRichTextString text351 = new HSSFRichTextString(outMemberVos.get(i).getUserDepartment());
            cell351.setCellValue(text351);
            cell351.setCellStyle(style10);

            HSSFCell cell352 = row1.createCell(2);
            HSSFRichTextString text352 = new HSSFRichTextString(outMemberVos.get(i).getWorkHour().toString());
            cell352.setCellValue(text352);
            cell352.setCellStyle(style10);
        }

        //准备将Excel的输出流通过response输出到页面下载
        //八进制输出流
        response.setContentType("application/octet-stream");

        //这后面可以设置导出Excel的名称，此例中名为student.xls
        response.setHeader("Content-disposition", "attachment;filename=work_management.xls");

        //刷新缓冲
        response.flushBuffer();

        //workbook将Excel写入到response的输出流中，供页面下载
        workbook.write(response.getOutputStream());
    }
}
