package com.xkrs.utils;

import com.xkrs.dao.ProjectOverviewDao;
import com.xkrs.dao.WorkHourRecordDao;
import com.xkrs.model.entity.ProjectOverview;
import com.xkrs.model.entity.WorkHourRecord;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Predicate;
import javax.persistence.criteria.Root;
import java.util.ArrayList;
import java.util.List;

/**
 * @author XinYi Song
 */

@Component
public class Query {

    @Resource
    private ProjectOverviewDao projectOverviewDao;

    @Resource
    private WorkHourRecordDao workHourRecordDao;

    /**
     * 动态多条件查询项目信息
     * @param
     * @return
     */
    public List<ProjectOverview> selectProjectByDynamic(String projectNumber, String projectName) {
        Specification<ProjectOverview> specification = new Specification<ProjectOverview>() {
            @Override
            public Predicate toPredicate(Root<ProjectOverview> root, CriteriaQuery<?> criteriaQuery, CriteriaBuilder criteriaBuilder) {
                List<Predicate> list = new ArrayList<>();
                if (projectNumber != null && !"".equals(projectNumber)) {
                    list.add(criteriaBuilder.equal(root.get("projectNumber").as(String.class), projectNumber));
                }
                if (projectName != null && !"".equals(projectName)) {
                    list.add(criteriaBuilder.like(root.get("projectName").as(String.class), "%" + projectName + "%"));
                }
                Predicate[] predicates = new Predicate[list.size()];
                return criteriaBuilder.and(list.toArray(predicates));
            }
        };
        return projectOverviewDao.findAll(specification);
    }


    /**
     * 查询工时记录
     * @param projectNumber
     * @param userId
     * @param startTime
     * @param endTime
     * @return
     */
    public List<WorkHourRecord> selectWorkRecord(String projectNumber, Integer userId, String startTime, String endTime) {
        Specification<WorkHourRecord> specification = new Specification<WorkHourRecord>() {
            @Override
            public Predicate toPredicate(Root<WorkHourRecord> root, CriteriaQuery<?> criteriaQuery, CriteriaBuilder criteriaBuilder) {
                List<Predicate> list = new ArrayList<>();
                if (projectNumber != null && !"".equals(projectNumber)) {
                    list.add(criteriaBuilder.equal(root.get("projectNumber").as(String.class), projectNumber));
                }
                if (userId != null && !"".equals(userId)) {
                    list.add(criteriaBuilder.equal(root.get("userId").as(Integer.class), userId));
                }
                list.add(criteriaBuilder.notEqual(root.get("color").as(String.class), "1"));
                if(startTime != null && !"".equals(startTime)){
                    list.add(criteriaBuilder.greaterThanOrEqualTo(root.get("submitTime").as(String.class), startTime));
                }
                if(endTime != null && !"".equals(endTime)){
                    list.add(criteriaBuilder.lessThanOrEqualTo(root.get("submitTime").as(String.class), endTime));
                }
                Predicate[] predicates = new Predicate[list.size()];
                return criteriaBuilder.and(list.toArray(predicates));
            }
        };
        return workHourRecordDao.findAll(specification);
    }


}